<?php

namespace Drupal\tab_manager\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Ajax\AjaxResponse;

/**
 * The form for management of tabs (local task links) on a current page.
 */
class TabManagerSettingsForm extends FormBase {

  /**
   * State service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * State service.
   *
   * @var \Drupal\Core\Routing\RouteBuilderInterface
   */
  protected $routerBuilder;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->state = $container->get('state');
    $instance->routerBuilder = $container->get('router.builder');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'tab_manager_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $current_tabs = $this->state->get('tab_manager_current_tabs');

    if (empty($current_tabs)) {
      $form['message'] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $this->t('There are no tabs on this page.'),
      ];
      return $form;
    }

    foreach ($current_tabs as $level => $tabs) {
      // Sorting the array by 'weight' so that the rows in tables are sorted.
      uasort($tabs, function ($a, $b) {
        return $a['weight'] <=> $b['weight'];
      });

      $form[$level] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        // Add 1 to start from 'Level: 1' for better user interface.
        '#value' => $this->t('Level: @level', ['@level' => ($level + 1)]),
      ];

      $name = 'tabs_level' . $level;
      $form[$name] = [
        '#type' => 'table',
        '#header' => [$this->t('Name'), $this->t('Weight'), ''],
        '#tabledrag' => [
          [
            'action' => 'order',
            'relationship' => 'sibling',
            'group' => 'table-sort-weight',
          ],
        ],
      ];
      foreach ($tabs as $key => $settings) {
        $form[$name][$key]['#attributes']['class'][] = 'draggable';
        $form[$name][$key]['#weight'] = $settings['weight'];
        $form[$name][$key]['title'] = [
          '#type' => 'textfield',
          '#default_value' => $settings['title'],
          '#size' => 20,
          '#required' => TRUE,
        ];

        $form[$name][$key]['weight'] = [
          '#type' => 'weight',
          '#title_display' => 'invisible',
          '#default_value' => $settings['weight'],
          '#attributes' => ['class' => ['table-sort-weight']],
        ];

        $form[$name][$key]['hidden'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('hide'),
          '#default_value' => $settings['hidden'],
        ];
      }
    }

    // ------------ Butons ----------------------------------------------------.
    $form['br'] = ['#type' => 'html_tag', '#tag' => 'br'];

    $form['actions']['save'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
    ];

    $form['actions']['reset_to_default'] = [
      '#type' => 'submit',
      '#value' => $this->t('Reset to Default'),
      '#button_type' => 'danger',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function ajaxSubmit(array &$form, FormStateInterface $form_state, Request $request) {
    return new AjaxResponse();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->getTriggeringElement()['#id'] == 'edit-save') {
      $this->save($form_state);
    }
    else {
      $this->resetToDefault($form_state);
    }
    // Flush routing and links cashes.
    $this->routerBuilder->rebuild();

    // Redirect to the page with tabs to force reloading and to see changes.
    $tabs_page_url = $this->state->get('tab_manager_tabs_page_url');
    $form_state->setRedirectUrl($tabs_page_url);
  }

  /**
   * {@inheritdoc}
   */
  private function save(FormStateInterface $form_state) {
    $current_tabs = $this->state->get('tab_manager_current_tabs');
    $levels_number = count($current_tabs);
    $modified_tabs = $this->state->get('tab_manager_modified_tabs');

    for ($i = 0; $i < $levels_number; $i++) {
      $tabs = $form_state->getValue('tabs_level' . $i);
      $modified_tabs = array_merge($modified_tabs, $tabs);
    }
    $this->state->set('tab_manager_modified_tabs', $modified_tabs);
  }

  /**
   * {@inheritdoc}
   */
  private function resetToDefault(FormStateInterface $form_state) {
    $current_tabs = $this->state->get('tab_manager_current_tabs');
    $modified_tabs = $this->state->get('tab_manager_modified_tabs');

    foreach ($current_tabs as $tabs) {
      $modified_tabs = array_diff_key($modified_tabs, $tabs);
    }

    $this->state->set('tab_manager_modified_tabs', $modified_tabs);
  }

}
